﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Robozzle
{
    public partial class Robozzle : Form
    {
        public Robozzle()
        {
            InitializeComponent();
            // Set the timeout to 0.5 seconds
            serialPort.ReadTimeout = 500;
            string error = "";
            try {
                serialPort.Open();
            } catch (Exception e) { error = e.Message;  }
            if (serialPort.IsOpen == false) {
                MessageBox.Show("Could not open Serial Port! " + error, "Serial Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                lbLog.Text = "NOT CONNECTED";
            }

            // Init the Robozzle map 80x25
            ColumnHeader[] cols = new ColumnHeader[25];
            int size = lstMap.Width / 25;
            for (int i = 0; i < 25; i++) {
                cols[i]       = new ColumnHeader();
                cols[i].Width = size;
                cols[i].Text  = " ";
            }
            lstMap.Columns.AddRange(cols);
            ListViewItem row;
            for (int i = 0; i < 80; i++) {
                row = new ListViewItem();
                row.UseItemStyleForSubItems = false;
                row.Text      = " ";
                row.BackColor = Color.Black;
                for (int j = 0; j < 25; j++) {
                    ListViewItem.ListViewSubItem col = new ListViewItem.ListViewSubItem();
                    col.Text = " "; col.BackColor = Color.Black;
                    row.SubItems.Add(col);
                }
                lstMap.Items.Add(row);
            }

            // Init the received buffer
            mBuffer = new int[4];
            mNoRecv = 0;

            // Testing...
            //test();
            lstMap.Refresh();
        }

        private void serialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            // PROTOCOL: 3bytes: row col type     - where type is in xX gG yY rR bB (uppercase meaning has a star)
            //        or 4bytes: row col 'Z' type - where type is x g y r b (back color of robozzle)
            try {
                while (true)
                {
                    mBuffer[mNoRecv] = serialPort.ReadByte();
                    count++;
                    lbLog.Text = "RECEIVED: " + count.ToString() + " BYTE:" + mBuffer[mNoRecv].ToString();
                    mNoRecv++;
                    if (mNoRecv >= 3)
                    {
                        // check if we need to receive one more by for robozzle background color
                        if ((mNoRecv == 3) && ((char)mBuffer[0] == 'Z'))
                        {
                            mBuffer[mNoRecv] = serialPort.ReadByte();
                            mNoRecv++;
                        }
                        addBox();
                        mNoRecv = 0;
                    }
                }
            } catch (Exception) {}
        }

        private void addBox()
        {
            lbLog.Text = "Add Box: Row:" + mBuffer[1].ToString() + " Col:" + mBuffer[2].ToString() + " Color:" + (char)mBuffer[0];
            ListViewItem.ListViewSubItem item = createBox((char)mBuffer[0]);
            ListViewItem row = lstMap.Items[mBuffer[1]];
            if (row == null)
                return;            
            row.SubItems[mBuffer[2]] = item;
        }

        private ListViewItem.ListViewSubItem createBox(char type)
        {
            ListViewItem.ListViewSubItem item = new ListViewItem.ListViewSubItem();
            item.Text = (Char.IsUpper(type)) ? "*" : " ";

again:
            type = Char.ToLower(type);
            switch (type)
            {
                case 'x':
                    item.BackColor = Color.Black;
                    break;
                case 'b':
                    item.BackColor = Color.Blue;
                    break;
                case 'g':
                    item.BackColor = Color.Green;
                    break;
                case 'r':
                    item.BackColor = Color.Red;
                    break;
                case 'a':
                    item.BackColor = Color.Gray;
                    break;
                case 'y':
                    item.BackColor = Color.Yellow;
                    break;
                case 'z':
                    item.Text = "Z";
                    type = (char)mBuffer[3];
                    goto again;
                default:
                    item.BackColor = Color.White;
                    break;
            }
            return item;
        }

        private void test()
        {
            string map = "xgxxBxxBxRBRRGRRRR";
            int norows = 6;
            int nocols = 3;
            int total  = norows * nocols;
            for (int i = 0; i < total; i++) {
                mBuffer[0] = i / nocols;
                mBuffer[1] = i % nocols;
                mBuffer[2] = map[i];
                addBox();
            }

            // add robozzle
            mBuffer[0] = 0;
            mBuffer[1] = 1;
            mBuffer[2] = 'Z';
            mBuffer[3] = 'g';
            addBox();
        }

        private int[] mBuffer;
        private int   mNoRecv;
        static int count = 0;
    }
}
